from __future__ import absolute_import

import logging
import logging.handlers
import inspect


def convert_loggers(configparser):
    names = configparser.get('loggers', 'keys')
    names = [x.strip() for x in names.split(',')]
    retval = {}
    for name in names:
        items = configparser.items('logger_{0}'.format(name), raw=True)
        section = {}
        final_name = name
        for key, value in items:
            if key == 'qualname':
                final_name = value
                continue
            if key == 'propagate':
                value = (value == '1')
            if key == 'handlers':
                value = [x.strip() for x in value.split()]
            section[key] = value
        retval[final_name] = section
    return retval


# Stolen from stdlib's logging.config
def _resolve(name):
    """Resolve a dotted name to a global object."""
    name = name.split('.')
    used = name.pop(0)
    found = __import__(used)
    for n in name:
        used = used + '.' + n
        try:
            found = getattr(found, n)
        except AttributeError:
            __import__(used)
            found = getattr(found, n)
    return found


def adapt_args(args, name):
    try:
        callable = _resolve(name)
    except ImportError:
        raise ValueError("unable to locate {0}".format(name))
    if inspect.isclass(callable):
        callable = callable.__init__
        skipself = True
    else:
        if not hasattr(callable, '__call__'):
            raise ValueError("{0} is not callable; can't process args".format(name))
        skipself = False
    argnames = inspect.getargspec(callable).args
    if skipself:
        if argnames[0] == 'self':
            argnames = argnames[1:]
    if len(argnames) < len(args):
        raise Exception("Too many args for {0}".format(name))
    return dict(zip(argnames, args))


def convert_handlers(configparser):
    names = configparser.get('handlers', 'keys')
    names = [x.strip() for x in names.split(',')]
    retval = {}
    for name in names:
        items = configparser.items('handler_{0}'.format(name), raw=True)
        section = {}
        args = tuple()
        for key, value in items:
            if key == 'args':
                args = eval(value, vars(logging))
                continue
            if key == 'class':
                klass = value
                continue
            section[key] = value
        try:
            # check in the logging namespace first
            eval(klass, vars(logging))
        except (AttributeError, NameError):
            section['class'] = klass
        else:
            section['class'] = 'logging.{0}'.format(klass)
        section.update(adapt_args(args, section['class']))
        retval[name] = section
    return retval


def convert_formatters(configparser):
    names = configparser.get('formatters', 'keys')
    names = [x.strip() for x in names.split(',')]
    retval = {}
    for name in names:
        items = configparser.items('formatter_{0}'.format(name), raw=True)
        section = {}
        retval[name] = section
        for key, value in items:
            if key == 'class':
                if value in ('Formatter', 'logging.Formatter'):
                    # default!
                    continue
                else:
                    key = '()'
            section[key] = value

    return retval


def combine(loggers, handlers, formatters):
    retval = {'version': 1}
    for k, v in loggers.items():
        if k == 'root':
            retval['root'] = v
        else:
            retval.setdefault('loggers', {})[k] = v
    retval['handlers'] = handlers
    retval['formatters'] = formatters
    return retval



from __future__ import absolute_import

import os.path
import pkg_resources
from characteristic import attributes
from .ini import IniConfigLoader
from .vendor import reify
from .structs import LoadableConfig, Loadable, loadable_type_entry_points
from .exceptions import ConfigNotFound


scheme_loadable_types = {
    'application': 'app',
    'composite': 'composite',
    'server': 'server',
    'filter': 'filter',
}


def lookup_object(spec):
    """
    Looks up a module or object from a some.module:func_name specification.
    To just look up a module, omit the colon and everything after it.
    """
    parts, target = spec.split(':') if ':' in spec else (spec, None)
    module = __import__(parts)

    for part in parts.split('.')[1:] + ([target] if target else []):
        module = getattr(module, part)

    return module


class CompositeHelper(object):
    def __init__(self, loader):
        self.loader = loader

    def get_app(self, name='main', global_conf=None):
        return self.loader.load_app(name)


@attributes(['path'], apply_with_init=False, apply_immutable=True)
class Loader(object):
    def __init__(self, path):
        self.path = path
        self.config_loader = Loader._find_config_loader(path)

    @staticmethod
    def _find_config_loader(config_path):
        suffix = _get_suffix(config_path)
        entry_point_name = "montague.config_loader"
        eps = tuple(pkg_resources.iter_entry_points(entry_point_name, suffix))
        if len(eps) == 0:
            # TODO log warning
            loader_cls = IniConfigLoader
        else:
            loader_cls = eps[0].load()
        return loader_cls(config_path)

    @reify
    def config(self):
        return self.config_loader.config()

    def _fallback_config_loader(self, schemes, kind, name):
        _configs = []
        global_config = self.config.get('globals', {})
        for scheme in schemes:
            scheme_config = self.config.get(scheme, {})
            if name in scheme_config:
                loadable_type = scheme_loadable_types[scheme]
                constructor = getattr(LoadableConfig, loadable_type)
                _configs.append(constructor(
                    name=name,
                    config=scheme_config[name],
                    global_config=global_config))
        if len(_configs) == 0:
            not_found = "the {0} {1}".format(kind, name)
            msg = "Unable to find the config for {0}".format(not_found)
            raise ConfigNotFound(msg)
        app_config = _configs[0]
        return app_config

    def app_config(self, name=None):
        try:
            if name is None:
                name = 'main'
            return self.config_loader.app_config(name)
        except (NotImplementedError, AttributeError):
            schemes = ['application', 'composite']
            app_config = self._fallback_config_loader(schemes, 'application', name)
            return app_config

    def _adapt_entry_point_factory(self, factory, entry_point_group):
        # TODO: use decorator or something to preserve signatures
        if entry_point_group in ['paste.composite_factory', 'paste.composit_factory']:
            def adapter(global_conf, **local_conf):
                helper = CompositeHelper(self)
                return factory(helper, global_conf, **local_conf)
            return adapter
        if entry_point_group in ['paste.server_runner', 'paste.filter_app_factory']:
            def outer(global_conf, **local_conf):
                def inner(wsgi_app):
                    return factory(wsgi_app, global_conf, **local_conf)
                return inner
            return outer
        return factory

    def _load_entry_point_factory(self, resource, entry_point_groups):
        pkg, name = resource.split('#')
        entry_point_map = pkg_resources.get_entry_map(pkg)
        factory = None
        for group in entry_point_groups:
            if group in entry_point_map:
                group_map = entry_point_map[group]
                if name in group_map:
                    factory = group_map[name].load()
                    factory = self._adapt_entry_point_factory(factory, group)
                    break
        if factory is None:
            raise Exception('TODO')
        return factory

    def _adapt_call_factory(self, factory, factory_type):
        # PasteDeploy assumes a call-type factory will meet the specification
        # for {type}_factory. That is, server_runner is not allowed.
        if factory_type == 'paste.composite_factory':
            def adapter(global_conf, **local_conf):
                helper = CompositeHelper(self)
                return factory(helper, global_conf, **local_conf)
            return adapter
        return factory

    def _load_call_factory(self, resource, factory_type):
        return self._adapt_call_factory(lookup_object(resource), factory_type)

    def _load_factory(self, location, factory_types):
        scheme, resource = location.split(':', 1)
        if scheme in ('egg', 'package'):
            factory = self._load_entry_point_factory(resource, factory_types)
        elif scheme == 'call':
            factory_type = [f for f in factory_types if f.endswith('_factory')][0]
            factory = self._load_call_factory(resource, factory_type)
        elif scheme == 'config':
            raise NotImplementedError("haven't implemented config delegation yet")
        else:
            raise NotImplementedError("assuming this is the 'import some code' type")
        return factory

    def _load_app_from_config(self, app_config):
        local_conf = dict(app_config.config)
        # resolve nested uses
        use = local_conf.pop('use')
        if ':' not in use:
            loadable = self._load_app(use)
            loadable.local_conf.update(local_conf)
            loadable.global_conf.update(app_config.global_config)
        else:
            factory = self._load_factory(use, app_config.entry_point_groups)
            loadable = Loadable(factory=factory, is_app=True,
                                global_conf=app_config.global_config,
                                local_conf=local_conf)
        filter_with = loadable.local_conf.pop('filter-with', None)
        loadable = loadable.normalize()
        if filter_with is not None:
            loadable.outer = self._load_filter(name=filter_with)
        return loadable

    def _load_app(self, name):
        if name is not None and name is not 'main' and ':' in name:
            # not a config section name, let's handle this
            factory = self._load_factory(name, loadable_type_entry_points['app'])
            loadable = Loadable(
                factory=factory, global_conf={}, local_conf={})
            return loadable
        app_config = self.app_config(name)
        return self._load_app_from_config(app_config)

    def load_app(self, name=None):
        loadable = self._load_app(name)
        return loadable.normalize().get()

    def server_config(self, name=None):
        try:
            if name is None:
                name = 'main'
            return self.config_loader.server_config(name)
        except (NotImplementedError, AttributeError):
            schemes = ['server']
            server_config = self._fallback_config_loader(schemes, 'server', name)
            return server_config

    def load_server(self, name=None):
        server_config = self.server_config(name)
        scheme, resource = server_config.config['use'].split(':', 1)
        if scheme in ('egg', 'package'):
            factory = self._load_entry_point_factory(
                resource, server_config.entry_point_groups)
        elif scheme == 'call':
            factory = self._load_call_factory(resource, server_config.loadable_type)
        else:
            raise Exception('TODO: scheme type {}'.format(scheme))
        local_conf = dict(server_config.config)
        del local_conf['use']
        return factory(server_config.global_config, **local_conf)

    def filter_config(self, name=None):
        try:
            if name is None:
                name = 'main'
            return self.config_loader.filter_config(name)
        except (NotImplementedError, AttributeError):
            schemes = ['filter']
            filter_config = self._fallback_config_loader(schemes, 'filter', name)
            return filter_config

    def _load_filter_from_config(self, filter_config):
        local_conf = dict(filter_config.config)
        # resolve nested uses
        use = local_conf.pop('use')
        if ':' not in use:
            loadable = self._load_filter(use)
            loadable.local_conf.update(local_conf)
            loadable.global_conf.update(filter_config.global_config)
        else:
            scheme, resource = filter_config.config['use'].split(':', 1)
            if scheme in ('egg', 'package'):
                factory = self._load_entry_point_factory(
                    resource, filter_config.entry_point_groups)
            elif scheme == 'call':
                factory = self._load_call_factory(resource, filter_config.loadable_type)
            else:
                raise Exception('TODO: scheme type {}'.format(scheme))
            loadable = Loadable(
                factory=factory,
                global_conf=filter_config.global_config,
                local_conf=local_conf)
        filter_with = loadable.local_conf.pop('filter-with', None)
        loadable = loadable.normalize()
        if filter_with is not None:
            loadable.outer = self._load_filter(name=filter_with)
        return loadable

    def _load_filter(self, name):
        if name is not None and name is not 'main' and ':' in name:
            # not a config section name, let's handle this
            factory = self._load_factory(name, loadable_type_entry_points['filter'])
            loadable = Loadable(
                factory=factory, global_conf={}, local_conf={})
            return loadable
        filter_config = self.filter_config(name)
        return self._load_filter_from_config(filter_config)

    def load_filter(self, name=None):
        return self._load_filter(name).normalize().get()

    def logging_config(self, name=None):
        if name is None:
            name = 'main'
        try:
            return self.config_loader.logging_config(name)
        except (NotImplementedError, AttributeError):
            return self.config['logging'][name]


def _get_suffix(path):
    basename = os.path.basename(path)
    return ".".join(basename.split(".")[1:])



from __future__ import absolute_import
"""This package contains files pulled from other projects to avoid
dependning on an entire project to get a single utility function."""
from .pyramid_decorator import reify

__all__ = ['reify']



from __future__ import absolute_import

import collections
import types


def validate_montague_standard_format(config):
    for key in ('globals', 'application', 'composite', 'filter', 'server', 'logging'):
        assert key in config
        assert isinstance(config[key], collections.Mapping)


def validate_config_loader_methods(config_loader):
    assert hasattr(config_loader, 'config')
    assert isinstance(config_loader.config, types.MethodType)
    specific_methods_required = False
    try:
        result = config_loader.config()
        validate_montague_standard_format(result)
    except NotImplementedError:
        # config loaders can fail to implement config() as long as they implement the other methods
        specific_methods_required = True

    for method in ('app_config', 'filter_config', 'server_config', 'logging_config'):
        if specific_methods_required:
            # If you don't implement .config(), you have to implement these
            assert hasattr(config_loader, method)
            assert isinstance(getattr(config_loader, method), types.MethodType)

        # We don't know the names of actual apps/filters/etc to load, but we do know
        # the loader shouldn't raise NotImplementedError if it has actually implemented them,
        # so we can try that.
        try:
            getattr(config_loader, method)('__bogus__')
        except NotImplementedError:
            if specific_methods_required:
                raise
        except Exception:
            # any other exception is fine here, because we don't know exactly what happens
            # with a bogus name. usually KeyError, but maybe something else would be raised
            pass



from __future__ import absolute_import

from itertools import tee
from six.moves import zip as izip
from characteristic import attributes, Attribute


loadable_type_entry_points = {
    'app': ['paste.app_factory'],
    'composite': ['paste.composite_factory', 'paste.composit_factory'],
    'server': ['paste.server_factory', 'paste.server_runner'],
    'filter': ['paste.filter_factory', 'paste.filter_app_factory'],
}


@attributes(['name', 'entry_point_groups', 'loadable_type',
             'config', 'global_config'], apply_immutable=True)
class LoadableConfig(object):
    @classmethod
    def app(cls, name, config, global_config):
        return cls(name=name, config=config, global_config=global_config,
                   loadable_type='app',
                   entry_point_groups=loadable_type_entry_points['app'])

    @classmethod
    def composite(cls, name, config, global_config):
        return cls(name=name, config=config, global_config={},
                   loadable_type='composite',
                   entry_point_groups=loadable_type_entry_points['composite'])

    @classmethod
    def server(cls, name, config, global_config):
        return cls(name=name, config=config, global_config={},
                   loadable_type='server',
                   entry_point_groups=loadable_type_entry_points['server'])

    @classmethod
    def filter(cls, name, config, global_config):
        return cls(name=name, config=config, global_config={},
                   loadable_type='filter',
                   entry_point_groups=loadable_type_entry_points['filter'])


@attributes(['filters'], apply_with_init=False)
class ComposedFilter(object):
    def __init__(self):
        self.filters = []

    def add_filter(self, filter):
        self.filters.append(filter)

    def __call__(self, app):
        for filter in self.filters:
            app = filter(app)
        return app


@attributes(['factory', 'local_conf', 'global_conf',
             Attribute('inner', default_value=None),
             Attribute('outer', default_value=None),
             Attribute('is_app', default_value=False)])
class Loadable(object):
    """A chain of Loadables is built from the config files. An app or filter with
       filter-with will become an Loadable with another Loadable as the 'outer'
       attribute, while a filter-app with a next property will become a Loadable
       with another Loadable as an 'inner' property."""
    def normalize(self):
        # The desired end state is a chain of loadables, starting with
        # the outermost filter, each with an inner property; the final
        # loadable contains the app (or last filter) and has no inner.

        # We call this method on what we can treat as the root loadable
        # of a binary tree, perform an in-order traversal to get the chain,
        # and then reconstruct the chain as (effectively) a linked list.

        chain = []
        self._make_chain(chain)
        for node in chain:
            node.inner = None
            node.outer = None
        for parent, child in pairwise(chain):
            parent.inner = child
        return chain[0]

    def _make_chain(self, chain):
        if self.outer is not None:
            self.outer._make_chain(chain)
        chain.append(self)
        if self.inner is not None:
            self.inner._make_chain(chain)

    def get(self):
        loaded = self.factory(self.global_conf, **self.local_conf)
        if self.inner is None:
            if self.is_app:
                return loaded
            else:
                # Need to compose these filters
                composed = ComposedFilter()
                composed.add_filter(loaded)
                return composed
        val = self.inner.get()
        if isinstance(val, ComposedFilter):
            # still composing
            val.add_filter(loaded)
            return val
        else:
            return loaded(val)


def pairwise(iterable):
    # recipe from itertools
    "s -> (s0,s1), (s1,s2), (s2, s3), ..."
    a, b = tee(iterable)
    next(b, None)
    return izip(a, b)



import os
import pytest
from six.moves.configparser import ConfigParser
from montague.logging import convert_loggers, convert_handlers, convert_formatters, combine
import sys

here = os.path.dirname(__file__)


@pytest.fixture
def ini_cp():
    ini_dir = os.path.join(here, 'config_files')
    ini_path = os.path.join(ini_dir, 'logging.ini')
    cp = ConfigParser()
    cp.read(ini_path)
    return cp
    

def test_loggers(ini_cp):
    expected = {
        'root': {
            'level': 'DEBUG',
            'handlers': ['console'],
        },
        'simpleExample': {
            'level': 'DEBUG',
            'handlers': ['console'],
            'propagate': False,
        },
    }
    actual = convert_loggers(ini_cp)
    assert expected == actual


def test_handlers(ini_cp):
    # py26 names it 'strm' for unknown reasons
    stream_key_name = 'stream' if (sys.version_info[0] > 2 or sys.version_info[1] > 6) else 'strm'
    expected = {
        'console': {
            'class': 'logging.StreamHandler',
            'level': 'DEBUG',
            'formatter': 'simple',
            stream_key_name: sys.stdout,
        },
        'syslog': {
            'class': 'logging.handlers.SysLogHandler',
            'level': 'WARNING',
            'formatter': 'complicated',
        }
    }
    actual = convert_handlers(ini_cp)
    assert expected == actual


def test_formatters(ini_cp):
    expected = {
        'simple': {
            'format': '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
        },
        'complicated': {
            'format': 'F1 %(asctime)s %(levelname)s %(message)s',
            'datefmt': '%a, %d %b %Y %H:%M:%S +0000',
            # class is omitted because it's the default
        },
        'withclass': {
            # montague doesn't have to inspect the callable for formatters
            # so this class doesn't have to exist
            '()': 'foobar.FooBarClass',
            'first': 'foo',
            'second': 'bar',
        },
    }
    actual = convert_formatters(ini_cp)
    assert expected == actual


def test_combined(ini_cp):
    loggers = convert_loggers(ini_cp)
    handlers = convert_handlers(ini_cp)
    formatters = convert_formatters(ini_cp)
    root_logger = loggers['root']
    other_loggers = dict([(k, v) for k, v in loggers.items() if k != 'root'])
    expected = {
        'version': 1,
        'root': root_logger,
        'loggers': other_loggers,
        'handlers': handlers,
        'formatters': formatters,
    }
    actual = combine(loggers, handlers, formatters)
    assert expected == actual



import sys


def main(argv=()):
    """
    Args:
        argv (list): List of arguments

    Returns:
        int: A return code

    Does stuff.
    """

    print(argv)
    return 0

if __name__ == "__main__":
    sys.exit(main())



from __future__ import absolute_import

from six.moves.configparser import SafeConfigParser, InterpolationError
from characteristic import attributes
from .structs import LoadableConfig
from .logging import convert_loggers, convert_handlers, convert_formatters, combine
import six
import os.path

SCHEMEMAP = {
    'application': 'application',
    'app': 'application',
    'composite': 'composite',
    'composit': 'composite',
    'server': 'server',
    'filter': 'filter',
    'filter-app': 'filter-app',
    'pipeline': 'pipeline'
}

LOGGING_SECTIONS = ('loggers', 'handlers', 'formatters')
MSF_KEYS = ('globals', 'application', 'composite', 'filter', 'server', 'logging')


@attributes(['path'], apply_with_init=False, apply_immutable=True)
class IniConfigLoader(object):
    """This config loader transforms a traditional INI file into
       a Montague Standard Format dictionary. It is compatible with
       most but not all PasteDeploy files."""

    def __init__(self, path):
        self.path = path
        self._data = self._read()
        self._config = self._process()

    def _read(self):
        # We need to keep the parser around so the logging conversion can use it.
        self._parser = SafeConfigParser()
        self._parser.read(self.path)
        self._globals = self._parser.defaults()
        data = {}
        for section in self._parser.sections():
            section_data = data.setdefault(section, {})
            for option in self._parser.options(section):
                if option in self._globals:
                    continue
                try:
                    section_data[option] = self._parser.get(section, option)
                except InterpolationError:
                    section_data[option] = self._parser.get(section, option, raw=True)
        return data

    def _process(self):
        orig = self._data
        config = {}
        for key in six.iterkeys(orig):
            if ':' in key:
                scheme, name = key.split(':', 1)
                kind_config = config.setdefault(SCHEMEMAP[scheme], {})
                kind_config[name] = orig[key]
            else:
                config[key] = orig[key]
        config['globals'] = {
            'here': os.path.dirname(self.path),
            '__file__': self.path,
        }
        for key, value in six.iteritems(self._globals):
            config['globals'][key] = value
        apps = config.setdefault('application', {})
        filters = config.setdefault('filter', {})
        generated_filter_count = 0
        filter_apps = config.pop('filter-app', {})
        for name, filter_app in six.iteritems(filter_apps):
            use = filter_app.pop('next')
            generated_filter_count += 1
            filter_name = '_montague_filter_{0}'.format(generated_filter_count)
            apps[name] = {'use': use, 'filter-with': filter_name}
            filters[filter_name] = filter_app
        pipelines = config.pop('pipeline', {})
        for name, pipeline in six.iteritems(pipelines):
            items = pipeline['pipeline'].split()
            pipeline_app = items[-1]
            pipeline_filters = items[:-1]
            pipeline_filters.reverse()
            apps[name] = {'use': pipeline_app}
            last_item = apps[name]
            for count, use_filter in enumerate(pipeline_filters, start=1):
                filter_name = '_montague_pipeline_{0}_filter_{1}'.format(name, count)
                filters[filter_name] = {'use': use_filter}
                last_item['filter-with'] = filter_name
                last_item = filters[filter_name]
        if all([self._parser.has_section(section_name) for section_name in LOGGING_SECTIONS]):
            loggers = convert_loggers(self._parser)
            handlers = convert_handlers(self._parser)
            formatters = convert_formatters(self._parser)

            config['logging'] = {'main': combine(loggers, handlers, formatters)}

        for key in MSF_KEYS:
            config.setdefault(key, {})
        return config

    def config(self):
        return self._config

    def app_config(self, name):
        # This method isn't actually necessary, since montague can extract
        # the config information from the MSF dict returned by .config()
        # but it's a nice example of how to do it.
        if name in self._config['application']:
            constructor = LoadableConfig.app
            local_config = self._config['application'][name]
        elif name in self._config['composite']:
            constructor = LoadableConfig.composite
            local_config = self._config['composite'][name]
        else:
            raise KeyError
        return constructor(
            name=name, config=local_config, global_config=self._config['globals'])

    def server_config(self, name):
        if name in self._config['server']:
            constructor = LoadableConfig.server
            local_config = self._config['server'][name]
        else:
            raise KeyError
        return constructor(
            name=name, config=local_config, global_config=self._config['globals'])

    def filter_config(self, name):
        if name in self._config['filter']:
            constructor = LoadableConfig.filter
            local_config = self._config['filter'][name]
        else:
            raise KeyError
        return constructor(
            name=name, config=local_config, global_config=self._config['globals'])

    def logging_config(self, name):
        # This is provided by .config(), so no need to implement it here.
        raise NotImplementedError



from __future__ import absolute_import

from .loadwsgi import Loader

__version__ = "0.2.0"


def load_app(config_path, name=None):
    loader = Loader(config_path)
    return loader.load_app(name)


def load_server(config_path, name=None):
    loader = Loader(config_path)
    return loader.load_server(name)


def load_filter(config_path, name=None):
    loader = Loader(config_path)
    return loader.load_filter(name)


def load_logging_config(config_path, name=None):
    loader = Loader(config_path)
    return loader.logging_config(name)



import os
import sys
import pytest
from montague.ini import IniConfigLoader
from montague.loadwsgi import Loader
from montague import load_app, load_server, load_filter, load_logging_config
from montague.structs import ComposedFilter
from montague.validation import validate_montague_standard_format, validate_config_loader_methods
import montague_testapps

here = os.path.dirname(__file__)


def test_read_config():
    ini_dir = os.path.join(here, 'config_files')
    ini_path = os.path.join(ini_dir, 'simple_config.ini')
    config = IniConfigLoader(ini_path)
    expected = {
        'globals': {
            'here': ini_dir,
            '__file__': ini_path,
            'foo': 'bar',
        },
        'application': {
            'main': {'use': 'package:montague_testapps#basic_app'},
            'egg': {'use': 'egg:montague_testapps#other'},
            'filtered-app': {
                'filter-with': 'filter',
                'use': 'package:montague_testapps#basic_app'
            },
        },
        'composite': {},
        'filter': {
            'filter': {
                'method_to_call': 'lower',
                'use': 'egg:montague_testapps#caps'
            },
            'filter1': {
                'filter-with': 'filter2',
                'use': 'egg:montague_testapps#caps'
            },
            'filter2': {
                'use': 'egg:montague_testapps#caps'
            },
        },
        'server': {
            'server_factory': {
                'port': '42',
                'use': 'egg:montague_testapps#server_factory',
            },
            'server_runner': {
                'host': '127.0.0.1',
                'use': 'egg:montague_testapps#server_runner'
            },
        },
        'logging': {
        },
    }
    assert config.config() == expected
    assert Loader(ini_path).config == expected


def test_load_app():
    config_path = os.path.join(here, 'config_files/simple_config.ini')
    app = load_app(config_path)
    app2 = load_app(config_path, name='egg')
    assert app is montague_testapps.apps.basic_app
    assert app2 is montague_testapps.apps.basic_app2


def test_load_server():
    config_path = os.path.join(here, 'config_files/simple_config.ini')
    server = load_server(config_path, name='server_factory')
    actual = server(montague_testapps.apps.basic_app)
    assert actual.montague_conf['local_conf']['port'] == '42'
    resp = actual.get('/')
    assert b'This is basic app' == resp.body
    server = load_server(config_path, name='server_runner')
    actual = server(montague_testapps.apps.basic_app2)
    assert actual.montague_conf['local_conf']['host'] == '127.0.0.1'
    resp = actual.get('/')
    assert b'This is basic app2' == resp.body


def test_load_filter():
    config_path = os.path.join(here, 'config_files/simple_config.ini')
    filter = load_filter(config_path, name='filter')
    app = filter(None)
    assert isinstance(app, montague_testapps.apps.CapFilter)


def test_load_filtered_app():
    config_path = os.path.join(here, 'config_files/simple_config.ini')
    app = load_app(config_path, name='filtered-app')
    assert isinstance(app, montague_testapps.apps.CapFilter)
    assert app.app is montague_testapps.apps.basic_app
    assert app.method_to_call == 'lower'


def test_load_layered_filter():
    config_path = os.path.join(here, 'config_files/simple_config.ini')
    filter = load_filter(config_path, name='filter1')
    assert isinstance(filter, ComposedFilter)
    app = filter(montague_testapps.apps.basic_app)
    assert app.app.app is montague_testapps.apps.basic_app
    assert isinstance(app, montague_testapps.apps.CapFilter)
    assert isinstance(app.app, montague_testapps.apps.CapFilter)


def test_filter_app():
    # Specifically the 'filter-app' config type
    config_path = os.path.join(here, 'config_files/filter_app.ini')
    app = load_app(config_path)
    assert app.method_to_call == 'lower'
    assert app.app.method_to_call == 'upper'
    assert app.app.app is montague_testapps.apps.basic_app


def test_null_logging_config():
    config_path = os.path.join(here, 'config_files/simple_config.ini')
    loader = Loader(config_path)
    with pytest.raises(KeyError):
        loader.logging_config()


def test_logging_config():
    stream_key_name = 'stream' if (sys.version_info[0] > 2 or sys.version_info[1] > 6) else 'strm'
    expected = {
        'version': 1,
        'root': {
            'level': 'DEBUG',
            'handlers': ['console'],
        },
        'loggers': {
            'simpleExample': {
                'level': 'DEBUG',
                'handlers': ['console'],
                'propagate': False,
            }
        },
        'handlers': {
            'console': {
                'class': 'logging.StreamHandler',
                'level': 'DEBUG',
                'formatter': 'simple',
                stream_key_name: sys.stdout,
            },
            'syslog': {
                'class': 'logging.handlers.SysLogHandler',
                'level': 'WARNING',
                'formatter': 'complicated',
            }
        },
        'formatters': {
            'simple': {
                'format': '%(asctime)s - %(name)s - %(levelname)s - %(message)s'
            },
            'complicated': {
                'format': 'F1 %(asctime)s %(levelname)s %(message)s',
                'datefmt': '%a, %d %b %Y %H:%M:%S +0000',
            },
            'withclass': {
                '()': 'foobar.FooBarClass',
                'first': 'foo',
                'second': 'bar',
            },
        },
    }
    config_path = os.path.join(here, 'config_files/logging.ini')
    actual = load_logging_config(config_path)
    assert actual == expected


def test_validity():
    config_path = os.path.join(here, 'config_files/simple_config.ini')
    config_loader = IniConfigLoader(config_path)
    validate_montague_standard_format(config_loader.config())
    validate_config_loader_methods(config_loader)



from __future__ import absolute_import

import json


class JSONConfigLoader(object):
    """This is the simplest possible config loader. It assumes
       the JSON document is already in Montague Standard Format,
       and that no adjustments or error checking is required."""

    def __init__(self, path):
        self.path = path

    def config(self):
        return json.load(open(self.path))



# -*- encoding: utf-8 -*-
import glob
import io
import re
from os.path import basename
from os.path import dirname
from os.path import join
from os.path import splitext

from setuptools import find_packages
from setuptools import setup


def read(*names, **kwargs):
    return io.open(
        join(dirname(__file__), *names),
        encoding=kwargs.get("encoding", "utf8")
    ).read()

setup(
    name="montague",
    version="0.2.0",
    license="MIT",
    description=("Montague provides functions to load WSGI apps and servers "
                 "based on configuration files. "
                 "It's based on PasteDeploy's INI file support."),
    long_description="%s\n%s" % (read("README.rst"),
                                 re.sub(":obj:`~?(.*?)`",
                                        r"``\1``", read("CHANGELOG.rst"))),
    author="Jon Rosebaugh",
    author_email="jon@inklesspen.com",
    url="https://github.com/inklesspen/montague",
    packages=find_packages("src"),
    package_dir={"": "src"},
    py_modules=[splitext(basename(i))[0] for i in glob.glob("src/*.py")],
    include_package_data=True,
    zip_safe=False,
    classifiers=[
        # complete classifier list: http://pypi.python.org/pypi?%3Aaction=list_classifiers
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: Unix",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: Implementation :: CPython",
        "Programming Language :: Python :: Implementation :: PyPy",
        "Topic :: Utilities",
    ],
    keywords=[
        # eg: "keyword1", "keyword2", "keyword3",
    ],
    install_requires=[
        # eg: "aspectlib==1.1.1", "six>=1.7",
        "six>=1.8.0",
        "zope.interface>=4.1.1",
        "characteristic>=14.2.0",
    ],
    extras_require={
        # eg: 'rst': ["docutils>=0.11"],
    },
    entry_points={
        "console_scripts": [
            "montague = montague.__main__:main"
        ]
    }

)



import os
import pytest
import pkg_resources
import mock
from montague.loadwsgi import Loader
from montague import load_app, load_server
import montague_testapps.apps
import montague.testjson
from montague.validation import validate_montague_standard_format, validate_config_loader_methods

here = os.path.dirname(__file__)


@pytest.yield_fixture(scope='function')
def working_set():
    # This is kind of a lot of work to go to just to get a dummy entry point.
    # Also plenty of pkg_resources methods use the global working set with no
    # way to override it.
    # Better APIs for this would be very nice.
    mock_ws = pkg_resources.WorkingSet()
    dist = pkg_resources.get_distribution('montague')
    mock_ep = pkg_resources.EntryPoint('json', 'montague.testjson', ['JSONConfigLoader'], dist=dist)
    mock_dist = pkg_resources.Distribution(project_name='montague_testjson', version='1.0')
    # There's some weird bug on Windows where if montague isn't installed
    # in develop mode, having mock_dist use dist's location won't work.
    mock_dist.location = montague.testjson.__file__
    mock_dist._ep_map = {
        'montague.config_loader': {
            'json': mock_ep
        }
    }
    mock_ws.add(mock_dist)
    patcher = mock.patch.multiple(pkg_resources,
                                  working_set=mock_ws,
                                  iter_entry_points=mock_ws.iter_entry_points,
                                  require=mock_ws.require)
    patcher.start()
    yield
    patcher.stop()


LOGGING_CONFIG = {
    u'loggers': {
        u'sqlalchemy.engine': {u'handlers': [], u'level': u'INFO'},
        u'mimir': {u'handlers': [], u'level': u'DEBUG'}
    },
    u'version': 1,
    u'root': {
        u'handlers': [u'console'],
        u'level': u'INFO'
    },
    u'formatters': {
        u'generic': {u'format': u'%(asctime)s %(levelname)-5.5s [%(name)s][%(threadName)s] %(message)s'}
    },
    u'handlers': {
        u'console': {
            u'formatter': u'generic',
            u'class': u'logging.StreamHandler',
            u'stream': u'ext://sys.stderr',
            u'level': u'NOTSET'
        }
    }
}


def test_read_config(working_set):
    expected = {
        "globals": {},
        'application': {
            'main': {'use': 'package:montague_testapps#basic_app'},
            'egg': {'use': 'egg:montague_testapps#other'},
            'filtered-app': {
                'filter-with': 'filter',
                'use': 'package:montague_testapps#basic_app',
            },
        },
        "composite": {},
        'filter': {
            'filter': {
                'use': 'egg:montague_testapps#caps',
                'method_to_call': 'lower',
            },
        },
        'server': {
            'server_factory': {
                'use': 'egg:montague_testapps#server_factory',
                'port': 42,
            },
            'server_runner': {
                'use': 'egg:montague_testapps#server_runner',
                'host': '127.0.0.1',
            },
        },
        'logging': {'main': LOGGING_CONFIG},
    }
    json_style_loader = Loader(os.path.join(here, 'config_files/simple_config.json'))

    assert json_style_loader.config_loader.config() == expected


def test_load_app(working_set):
    config_path = os.path.join(here, 'config_files/simple_config.json')
    app = load_app(config_path)
    app2 = load_app(config_path, name='egg')
    assert app is montague_testapps.apps.basic_app
    assert app2 is montague_testapps.apps.basic_app2


def test_load_server(working_set):
    config_path = os.path.join(here, 'config_files/simple_config.json')
    server = load_server(config_path, name='server_factory')
    actual = server(montague_testapps.apps.basic_app)
    assert actual.montague_conf['local_conf']['port'] == 42
    resp = actual.get('/')
    assert b'This is basic app' == resp.body
    server = load_server(config_path, name='server_runner')
    actual = server(montague_testapps.apps.basic_app2)
    assert actual.montague_conf['local_conf']['host'] == '127.0.0.1'
    resp = actual.get('/')
    assert b'This is basic app2' == resp.body


def test_load_filtered_app(working_set):
    config_path = os.path.join(here, 'config_files/simple_config.json')
    app = load_app(config_path, name='filtered-app')
    assert isinstance(app, montague_testapps.apps.CapFilter)
    assert app.app is montague_testapps.apps.basic_app
    assert app.method_to_call == 'lower'


def test_load_logging_config(working_set):
    config_path = os.path.join(here, 'config_files/simple_config.json')
    loader = Loader(config_path)
    actual = loader.logging_config()
    assert actual == LOGGING_CONFIG


def test_validity(working_set):
    loader = Loader(os.path.join(here, 'config_files/simple_config.json'))
    config_loader = loader.config_loader
    validate_montague_standard_format(config_loader.config())
    validate_config_loader_methods(config_loader)
